// //////////////////////////////////////////////////////////////////////////
//
//     nmer_db.h
//     (c) 2007 Frank DiMaio
//
// //////////////////////////////////////////////////////////////////////////
//
//     This file is part of ACMI.
// 
//     ACMI is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 3 of the License, or
//     (at your option) any later version.
// 
//     ACMI is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ////////////////////////////////////////////////////////////////////////////


#include <string>
#include <map>
#include <iostream>
#include <sys/dir.h>
#include <vector>
#include <fstream>

#include "chem.h"


using namespace std;


// does file exist?
bool exists(string filename) {
	fstream fin;
	fin.open(filename.c_str(),ios::in);
	if( fin.is_open() )
		return true;
	return false;
}


//
//
struct nmerDBEntry {
	int  fileId;
	char chainId;
	int  resId;
};
	
void DB_dump(string outfile, map < int,vector<nmerDBEntry> > &DB_nmer , vector < string > &DB_filename) {
	ofstream out;
	out.open(outfile.c_str(), ios::out);
	
	// database
	int nnmers = DB_nmer.size();
	out.write( (char*)&nnmers , sizeof(int) );
	for (map< int,vector<nmerDBEntry> >::iterator p = DB_nmer.begin(); p!= DB_nmer.end(); ++p) {
		int key = p->first;
		out.write( (char*)&key , sizeof(int) );
		
		vector<nmerDBEntry> &nmerlist = p->second;
		int nentries = nmerlist.size();
		out.write( (char*)&nentries , sizeof(int) );
		out.write( (char*)&nmerlist[0], nentries*sizeof(nmerDBEntry) );
	}
		
	// filelist
	int nfiles = DB_filename.size();
	out.write( (char*)&nfiles , sizeof(int) );
	for (int i=0; i<nfiles; ++i) {
		int filenamelen = DB_filename[i].size();
		out.write( (char*)&filenamelen, sizeof(int) );
		out.write( (char*)DB_filename[i].c_str() ,(filenamelen+1)*sizeof(char) );  // +1 for null terminator
	}
	
	DB_nmer.clear();
	DB_filename.clear();
}

void DB_load(string infile, map < int,vector<nmerDBEntry> > &DB_nmer , vector < string > &DB_filename) {
	ifstream in;
	in.open(infile.c_str(), ios::in);
	DB_nmer.clear();
	DB_filename.clear();
	
	if (!in.is_open()) {
		cerr << "ERROR: Unable to open nmer DB " << infile << endl;
		cerr << "Is the nmer database installed?  Is the environment variable \'ACMI_NMERDB\' set?" << endl;
		exit(1);
	} 
	
	// database
	int nnmers=-1;
	in.read( (char*)&nnmers , sizeof(int) );
	//DB_nmer.resize(nnmers);
	for (int i=0; i<nnmers; ++i) {
		int key=-1;
		in.read( (char*)&key , sizeof(int) );
		
		vector<nmerDBEntry> nmerlist;
		int nentries=-1;
		in.read( (char*)&nentries , sizeof(int) );
		nmerlist.resize(nentries);
		in.read( (char*)&nmerlist[0], nentries*sizeof(nmerDBEntry) );
		
		DB_nmer[key] = nmerlist;
	}

	// filelist
	int nfiles=-1;
	in.read( (char*)&nfiles , sizeof(int) );
	DB_filename.resize(nfiles);
	char charbuffer[1024];
	for (int i=0; i<nfiles; ++i) {
		int filenamelen=-1;
		in.read( (char*)&filenamelen, sizeof(int) );
		if (filenamelen>1023) { cerr << "Corrupted DB_filename!\n"; exit(1); }
		in.read( (char*)charbuffer , (filenamelen+1)*sizeof(char) );  // +1 for null terminator
		DB_filename[i] = string(charbuffer);
	}
}

